<?php
/**
 * Order repository class
 */
class DataAccess_OrderRepository implements DataAccess_OrderRepositoryInterface
{
	protected $db;
	protected $settings;

	/**
	 * Class constructor
	 *
	 * @param DB $db
	 * @param null $settings
	 */
	public function __construct(DB $db = null, &$settings = null)
	{
		if (is_null($db))
		{
			global $db;
		}

		if (is_null($settings))
		{
			global $settings;
		}

		$this->db = $db;
		$this->settings = $settings;
	}

	/**
	 * Get last active order
	 *
	 * @param string $dateFormat
	 * @param int $count
	 *
	 * @return array
	 */
	public function getLastActiveOrders($dateFormat = '%m/%d/%Y - %r', $count = 7)
	{
		return $this->db->selectAll('
			SELECT
				'.DB_PREFIX.'orders.*, '.DB_PREFIX.'users.*, DATE_FORMAT(status_date, "'.$dateFormat.'") AS sd
			FROM '.DB_PREFIX.'orders
			LEFT JOIN '.DB_PREFIX.'users ON
				'.DB_PREFIX.'users.uid='.DB_PREFIX.'orders.uid
				AND '.DB_PREFIX.'users.removed="No"
			WHERE
				'.DB_PREFIX.'orders.status <> "Abandon"
				AND '.DB_PREFIX.'orders.status <> "New"
				AND order_num > 0
				AND '.DB_PREFIX.'orders.status <> "Failed"
				AND '.DB_PREFIX.'orders.removed <> "Yes"
			ORDER BY '.DB_PREFIX.'orders.placed_date DESC
			LIMIT '.intval($count).'
		');
	}

	/**
	 * Reset order line item's shipping price
	 *
	 * @param $orderId
	 */
	public function resetLineItemShippingPrices($orderId)
	{
		$this->db->reset();
		$this->db->assignStr('is_shipping_price', 'No');
		$this->db->assignStr('shipping_price', '0.00');
		$this->db->update(DB_PREFIX.'orders_content', 'WHERE oid = '.intval($orderId));
	}

	/**
	 * Reset line item discounts
	 *
	 * @param int $orderId
	 */
	public function resetLineItemDiscounts($orderId)
	{
		$this->db->query('UPDATE '.DB_PREFIX.'orders_content SET discount_amount = 0, promo_discount_amount = 0 WHERE oid = '.intval($orderId));
	}

	/**
	 * Update line item discount
	 *
	 * @param int $orderId
	 * @param int $lineId
	 * @param float $discountAmount
	 * @param float $promoDiscountAmount
	 */
	public function updateLineItemDiscount($orderId, $lineId, $discountAmount, $promoDiscountAmount)
	{
		$this->db->query('
			UPDATE '.DB_PREFIX.'orders_content
			SET discount_amount = '.floatval($discountAmount).', promo_discount_amount = '.floatval($promoDiscountAmount).'
			WHERE oid = '.intval($orderId).' AND ocid = '.intval($lineId)
		);
	}

	/**
	 * Find existing line item for a product
	 *
	 * @param ORDER $order
	 * @param $productId
	 * @param $options
	 *
	 * @return mixed
	 */
	public function findExistingLineItemForProduct(ORDER $order, $productId, $options)
	{
		$lineItemValues = $this->db->selectOne('
			SELECT
				oc.ocid, p.price AS final_price
			FROM '.DB_PREFIX.'orders_content oc
			INNER JOIN '.DB_PREFIX.'products p ON oc.pid = p.pid
			WHERE
				oc.oid = '.$order->getId().'
				AND oc.product_id = "'.$this->db->escape($productId).'"
				AND oc.is_gift = "No"
				AND oc.options LIKE "'.$this->db->escape($options).'"',
			MYSQL_ASSOC
		);

		if ($lineItemValues)
		{
			if (isset($order->lineItems[$lineItemValues['ocid']]))
			{
				$lineItem = $order->lineItems[$lineItemValues['ocid']];
				$lineItem->setFinalPrice($lineItemValues['final_price']);

				return $lineItem;
			}
		}

		return null;
	}

	/**
	 * Get order items products
	 *
	 * @param int $orderId
	 * @param array $lineItemIds
	 *
	 * @return mixed
	 */
	public function getOrderItemsProducts($orderId, &$lineItemIds)
	{
		if (count($lineItemIds) == 0) return array();

		foreach ($lineItemIds as $key => $lineItemId)
		{
			$lineItemIds[$key] = intval($lineItemId);
		}

		$productsTemp = $this->db->selectAll('
			SELECT DISTINCT p.*
			FROM '.DB_PREFIX.'orders_content AS oc
			INNER JOIN '.DB_PREFIX.'products p ON oc.pid = p.pid
			WHERE
				oc.ocid IN ('.implode(',', $lineItemIds).')
				AND oc.oid = '.intval($orderId)
		);

		$products = array();
		foreach ($productsTemp as $prod)
		{
			$products[$prod['pid']] = new Model_Product($prod);
		}

		return $products;
	}

	/**
	 * Delete line items
	 *
	 * @param $orderId
	 * @param $lineItemIds
	 */
	public function deleteLineItems($orderId, &$lineItemIds)
	{
		foreach ($lineItemIds as $key => $lineItemId)
		{
			$lineItemIds[$key] = intval($lineItemId);
		}

		if (count($lineItemIds) > 0)
		{
			$this->db->query('DELETE FROM '.DB_PREFIX.'orders_content WHERE oid = '.intval($orderId).' AND ocid IN ('.implode(',', $lineItemIds).')');
			$this->db->query('DELETE FROM '.DB_PREFIX.'orders_content_attributes WHERE oid = '.intval($orderId).' AND ocid IN ('.implode(',', $lineItemIds).')');
		}
	}

	/**
	 * Delete all line items
	 *
	 * @param $orderId
	 */
	public function deleteAllLineItems($orderId)
	{
		$this->db->query('DELETE FROM '.DB_PREFIX.'orders_content WHERE oid = '.intval($orderId));
		$this->db->query('DELETE FROM '.DB_PREFIX.'orders_content_attributes WHERE oid = '.intval($orderId));
	}

	/**
	 * Persis line item
	 *
	 * @param ORDER $order
	 * @param Model_LineItem $lineItemObj
	 * @param bool $updateAttributes
	 */
	public function persistLineItem(ORDER $order, Model_LineItem $lineItemObj, $updateAttributes = false)
	{
		$this->db->reset();

		$lineItem = $lineItemObj->toArray();

		if (!isset($lineItem['is_gift']) || $lineItem['is_gift'] == 'No')
		{
			$this->db->assign('price', PriceHelper::round($lineItem['price']));
			$this->db->assign('admin_price', PriceHelper::round($lineItem['admin_price']));
			$this->db->assign('price_before_quantity_discount', PriceHelper::round($lineItem['price_before_quantity_discount']));
			$this->db->assignStr('free_shipping', $lineItem['free_shipping']);
			$this->db->assignStr('is_gift', 'No');
		}
		else
		{
			$this->db->assign('price', 0);
			$this->db->assign('admin_price', 0);
			$this->db->assignStr('is_gift', 'Yes');
		}

		$this->db->assign('cost', $lineItem['cost']);
		$this->db->assign('quantity', $lineItem['quantity']);
		$this->db->assign('admin_quantity', $lineItem['admin_quantity']);
		$this->db->assign('weight', $lineItem['weight']);

		$this->db->assignStr('is_taxable', $lineItem['is_taxable']);
		$this->db->assignStr('tax_rate', $lineItem['tax_rate']);
		$this->db->assignStr('tax_description', $lineItem['tax_description']);
		$this->db->assign('price_withtax', PriceHelper::round($lineItem['price_withtax']));

		$this->db->assignStr('product_id', $lineItem['product_id']);
		$this->db->assignStr('product_sku', $lineItem['product_sku']);
		$this->db->assignStr('product_upc', $lineItem['product_upc']);
		$this->db->assignStr('product_gtin', $lineItem['product_gtin']);
		$this->db->assignStr('product_mpn', $lineItem['product_mpn']);

		$this->db->assignStr('title', $lineItem['title']);

		// TODO - possibly need some kind of flag here or so - like for attributes
		if (isset($lineItem['options'])) $this->db->assignStr('options', $lineItem['options']);
		if (isset($lineItem['options_clean'])) $this->db->assignStr('options_clean', $lineItem['options_clean']);
		if (isset($lineItem['product_sub_id'])) $this->db->assignStr('product_sub_id', $lineItem['product_sub_id']);
		if (isset($lineItem['inventory_id'])) $this->db->assignStr('inventory_id', $lineItem['inventory_id']);
		if (isset($lineItem['products_location_id'] )) $this->db->assignStr('products_location_id', $lineItem['products_location_id']);

		if (isset($lineItem['ocid']) && intval($lineItem['ocid']) > 0)
		{
			$this->db->update(DB_PREFIX.'orders_content', 'WHERE ocid='.intval($lineItem['ocid']).' AND oid='.intval($order->getId()).' AND pid='.intval($lineItem['pid']));

			// remove attributes before update
			if ($updateAttributes)
			{
				$this->db->query('DELETE FROM '.DB_PREFIX.'orders_content_attributes WHERE ocid='.intval($lineItem['ocid']));
			}
		}
		else
		{
			$this->db->assign('oid', $order->getId());
			$this->db->assign('pid', $lineItem['pid']);

			// some of properties set only when added
			$this->db->assignStr('is_dollar_days', $lineItem['is_dollar_days']);
			$this->db->assignStr('is_doba', $lineItem['is_doba']);
			$this->db->assignStr('digital_product', $lineItem['digital_product']);
			$this->db->assignStr('digital_product_file', $lineItem['digital_product_file']);
			$this->db->assignStr('digital_product_key', $lineItem['digital_product'] == 'Yes' ? md5(uniqid(rand(),1)) : '');
			$this->db->assignStr('digital_product_downloads', '0');

			$this->db->assign('discount_amount', $lineItem['discount_amount']);
			$this->db->assign('promo_discount_amount', $lineItem['promo_discount_amount']);

			$lineItemObj->setId($this->db->insert(DB_PREFIX.'orders_content'));
			$lineItem['ocid'] = $lineItemObj->getId();
		}

		// store/update attributes
		if ($updateAttributes && isset($lineItem['attributes']) && is_array($lineItem['attributes']) && count($lineItem['attributes']) > 0)
		{
			for ($i=0; $i<count($lineItem['attributes']); $i++)
			{
				if ($lineItem['attributes'][$i]['attribute_value'] * 1 != 0)
				{
					$this->db->reset();
					$this->db->assignStr('ocid', $lineItem['ocid']);
					$this->db->assignStr('oid', $order->getId());
					$this->db->assignStr('attribute_value', $lineItem['attributes'][$i]['attribute_value']);
					$this->db->assignStr('attribute_value_type', $lineItem['attributes'][$i]['attribute_value_type']);
					$this->db->insert(DB_PREFIX.'orders_content_attributes');
				}
			}
		}
	}

	/**
	 * Increment payment attempts
	 *
	 * @param $orderId
	 * @param $limit
	 *
	 * @return bool
	 */
	public function incrementPaymentAttempt($orderId, $limit)
	{
		$this->db->query('SELECT attempts FROM '.DB_PREFIX.'orders WHERE oid = '.intval($orderId));

		if ($this->db->moveNext())
		{
			$a = $this->db->col['attempts'];
			if ($a >= $limit)
			{
				$this->db->reset();
				$this->db->assignStr('status', 'Failed');
				$this->db->assignStr('payment_is_realtime', 'Yes');
				$this->db->assignStr('payment_status', 'Error');
				$this->db->assign('status_date', 'NOW()');
				$this->db->update(DB_PREFIX.'orders', 'WHERE oid = '.intval($orderId));

				return false;
			}
			else
			{
				$this->db->reset();
				$this->db->assign('attempts', 'attempts + 1');
				$this->db->update(DB_PREFIX.'orders', 'WHERE oid = '.intval($orderId));
			}
		}

		return true;
	}

	/**
	 * Get next order number
	 *
	 * @param $orderId
	 * @param $minOrderNumber
	 *
	 * @return int
	 */
	public function getNextOrderNumber($orderId, $minOrderNumber)
	{
		$this->db->query('SELECT order_num FROM '.DB_PREFIX.'orders WHERE oid = '.intval($orderId));

		if ($this->db->moveNext())
		{
			if ($this->db->col['order_num'] < 1)
			{
				/**
				 * Set order num - all in one query to avoid duplicates
				 */
				$min = intval($minOrderNumber);
				$q = '
					UPDATE '.DB_PREFIX.'orders
					SET
						order_num = (
							SELECT * FROM (
								SELECT IF(MAX(order_num) >= '.intval($min).', MAX(order_num) + 1, '.intval($min).')
								FROM '.DB_PREFIX.'orders AS o
							) AS new_order_num
						)
					WHERE oid='.intval($orderId);

				$this->db->query($q);

				/**
				 * Return it
				 */
				$this->db->query('SELECT order_num FROM '.DB_PREFIX.'orders WHERE oid = '.intval($orderId));
				$this->db->moveNext();
			}

			return $this->db->col['order_num'];
		}

		return 0;
	}

	/**
	 * Check is shipping digital
	 *
	 * @param int $orderId
	 *
	 * @return int
	 */
	public function checkShippingDigital($orderId)
	{
		$result = $this->db->selectOne('
			SELECT count(*) AS c
			FROM '.DB_PREFIX.'orders_content
			WHERE digital_product="No" AND product_id <> "gift_certificate" AND oid = '.intval($orderId),
			MYSQL_ASSOC
		);

		$shippingDigital = $result['c'] > 0 ? 0 : 1;

		$this->db->reset();

		if ($shippingDigital == 1)
		{
			$this->db->assignStr('shipping_carrier_id', 'custom');
			$this->db->assignStr('shipping_method_id', 'digital');
			$this->db->assignStr('shipping_cm_name', 'Digital Download');
			$this->db->assignStr('shipping_taxable', 0);
			$this->db->assignStr('shipping_tax_rate', 0);
			$this->db->assignStr('shipping_tax_description', '');
			$this->db->assignStr('shipping_tax_amount', '0.00');
		}

		$this->db->assignStr('shipping_digital', $shippingDigital);
		$this->db->update(DB_PREFIX.'orders', 'WHERE oid = '.intval($orderId));

		return $shippingDigital;
	}

	/**
	 * Get shipping address
	 *
	 * @param $orderId
	 *
	 * @return array|mixed
	 */
	public function getShippingAddress($orderId)
	{
		$addressData = $this->db->selectOne('
			SELECT
				'.DB_PREFIX.'orders.shipping_address_type AS address_type,
				'.DB_PREFIX.'orders.shipping_name AS name,
				'.DB_PREFIX.'orders.shipping_company AS company,
				'.DB_PREFIX.'orders.shipping_address1 AS address1,
				'.DB_PREFIX.'orders.shipping_address2 AS address2,
				'.DB_PREFIX.'orders.shipping_city AS city,
				'.DB_PREFIX.'orders.shipping_zip AS zip,
				'.DB_PREFIX.'orders.shipping_province AS province,
				'.DB_PREFIX.'orders.shipping_state AS state_id,
				'.DB_PREFIX.'countries.coid AS country_id,
				'.DB_PREFIX.'countries.name AS country,
				'.DB_PREFIX.'countries.name AS country_name,
				'.DB_PREFIX.'countries.iso_a2 AS country_iso_a2,
				'.DB_PREFIX.'countries.iso_a3 AS country_iso_a3,
				'.DB_PREFIX.'countries.iso_number AS country_iso_number,
				'.DB_PREFIX.'states.short_name AS state_abbr,
				'.DB_PREFIX.'states.name AS state
			FROM '.DB_PREFIX.'orders
			LEFT JOIN '.DB_PREFIX.'countries ON '.DB_PREFIX.'countries.coid = '.DB_PREFIX.'orders.shipping_country
			LEFT JOIN '.DB_PREFIX.'states ON '.DB_PREFIX.'states.stid = '.DB_PREFIX.'orders.shipping_state
			WHERE oid = '.intval($orderId), MYSQL_ASSOC);

		return $addressData ? $addressData : array();
	}

	/**
	 * Get line items
	 *
	 * @param $orderId
	 *
	 * @return mixed
	 */
	public function getLineItems($orderId)
	{
		return $this->db->selectAll('
			SELECT
				p.*,
				IF(p.url_custom = "", p.url_default, p.url_custom) AS product_url,
				oc.product_sub_id,
				oc.ocid,
				oc.admin_price AS product_price,
				oc.admin_price,
				oc.price,
				oc.options,
				oc.options_clean,
				oc.quantity,
				oc.admin_quantity,
				oc.is_taxable,
				oc.tax_rate,
				oc.tax_description,
				oc.tax_exempt,
				oc.weight AS product_weight,
				oc.is_gift,
				oc.free_shipping,
				oc.digital_product_key,
				oc.price_before_quantity_discount,
				c.name AS cat_name,
				IF(c.url_custom = "", c.url_default, c.url_custom) AS cat_url,
				c.exactor_euc_code AS cat_exactor_euc_code,
				oc.discount_amount,
				oc.promo_discount_amount,
				oc.price_withtax,
				oc.tax_amount,
				oc.inventory_id
			FROM '.DB_PREFIX.'orders_content AS oc
			INNER JOIN '.DB_PREFIX.'products AS p ON p.pid = oc.pid
			LEFT JOIN '.DB_PREFIX.'catalog AS c ON p.cid = c.cid
			WHERE
				oc.oid = '.intval($orderId).'
			ORDER BY ocid
		');
	}

	/**
	 * Get order data
	 *
	 * @param $orderId
	 *
	 * @return mixed
	 */
	public function getOrderData($orderId)
	{
		return $this->db->selectOne('
			SELECT *, DATE_FORMAT(status_date, "'.$this->settings['LocalizationDateTimeFormat'].'") as status_date_formatted
			FROM '.DB_PREFIX.'orders WHERE oid = '.intval($orderId),
			MYSQL_ASSOC
		);
	}

	/**
	 * Persist order data
	 *
	 * @param ORDER $order
	 */
	public function persistOrderData(ORDER $order)
	{
		$db = $this->db;

		$db->reset();
		// Subtotal
		$db->assignStr('subtotal_amount', $order->getSubtotalAmount());

		$db->assign('force_recalc', $order->getForceRecalc() ? 1 : 0);

		// Discounts
		$db->assignStr('discount_value', $order->getDiscountValue());
		$db->assignStr('discount_type', $order->getDiscountType());
		$db->assignStr('discount_amount', $order->getDiscountAmount());

		// Promo Discounts
		$db->assignStr('promo_discount_amount', $order->getPromoDiscountAmount());
		$db->assignStr('promo_discount_value', $order->getPromoDiscountValue());
		$db->assignStr('promo_discount_type', $order->getPromoDiscountType());
		$db->assignStr('promo_type', $order->getPromoType());
		$db->assignStr('promo_campaign_id', $order->getPromoCampaignId());

		// Handling
		$db->assignStr('handling_separated', $order->getHandlingSeparated() ? '1' : '0');
		$db->assignStr('handling_fee', $order->getHandlingFee());
		$db->assignStr('handling_fee_type', $order->getHandlingFeeType());
		$db->assignStr('handling_fee_when_free_shipping', $order->getHandlingFeeWhenShippingFree());
		$db->assignStr('handling_text', $order->getHandlingText());
		$db->assignStr('handling_amount', $order->getHandlingAmount());
		$db->assignStr('handling_taxable', $order->getHandlingTaxable() ? '1' : '0');
		$db->assignStr('handling_tax_rate', $order->getHandlingTaxRate());
		$db->assignStr('handling_tax_amount', $order->getHandlingTaxAmount());
		$db->assignStr('handling_tax_description', $order->getHandlingTaxDescription());

		//TODO: This isn't persisted anywhere
		//$db->assignStr('handling_tax_class_id', $order->getHandlingTaxClassId());

		// Shipping
		$db->assignStr('shipping_ssid', $order->getShippingSSID());
		$db->assignStr('shipping_carrier_id', $order->getShippingCarrierId());
		$db->assignStr('shipping_method_id', $order->getShippingMethodId());
		$db->assignStr('shipping_cm_name', $order->getShippingCMName());
		$db->assignStr('shipping_taxable', $order->getShippingTaxable() ? '1' : '0');
		$db->assignStr('shipping_tax_rate', $order->getShippingTaxRate());
		$db->assignStr('shipping_tax_description', $order->getShippingTaxDescription());
		$db->assignStr('shipping_tax_amount', $order->getShippingTaxAmount());
		$db->assignStr('shipping_amount', $order->shippingAmount);
		$db->assignStr('shipping_ignore_free', $order->getShippingIgnoreFree() ? '1' : '0');

		$db->assign('shipping_digital', $order->getShippingDigital());

		$db->assignStr('shipping_tracking_number', $order->getShippingTrackingNumber());
		$db->assignStr('shipping_tracking_number_type', $order->getShippingTrackingNumberType());

		$db->assign('is_mobile', intval($order->getIsMobile()));
		$db->assign('offsite_campaign_id', intval($order->getOffsiteCampaignId()));
		$db->assignStr('order_source', $order->getOrderSource());

		$shipping_tracking = $order->getShippingTracking();

		if (count($shipping_tracking) == 0)
		{
			$db->assignStr('shipping_tracking', '');
		}
		else
		{
			$db->assignStr('shipping_tracking', serialize($shipping_tracking));
		}

		// Tax
		$db->assignStr('tax_amount', $order->getTaxAmount());
		$db->assignStr('tax_exempt', $order->getTaxExempt());

		// Total
		$db->assignStr('total_amount', $order->getTotalAmount());

		// TODO: Persist these values
		// $db->assignStr('status', $order->getStatus());
		// $db->assignStr('payment_status', $order->getPaymentStatus());

		$db->update(DB_PREFIX.'orders', 'WHERE oid='.intval($order->getId()));
	}

	/**
	 * Update line item quantity pricing
	 *
	 * @param $orderId
	 * @param $lineItemId
	 * @param $finalPrice
	 * @param $priceWithTax
	 * @param $freeShipping
	 */
	public function updateLineItemQuantityPricing($orderId, $lineItemId, $finalPrice, $priceWithTax, $freeShipping)
	{
		$db = $this->db;

		$db->reset();
		$db->assign('price', $finalPrice);
		$db->assign('admin_price', $finalPrice);
		$db->assign('price_withtax', $priceWithTax);
		$db->assignStr('free_shipping', $freeShipping);
		$db->update(DB_PREFIX.'orders_content', 'WHERE oid = '.intval($orderId).' AND ocid = '.intval($lineItemId));
	}

	/**
	 * Get product gifts data
	 *
	 * @param int $orderId
	 * @return mixed
	 */
	public function getProductsGiftsData($orderId)
	{
		return $this->db->selectAll('
			SELECT p.*,
				g.gift_quantity,
				g.gift_shipping_free,
				g.gift_max_quantity,
				p2.pid as original_pid,
				p2.gift_quantity as gift_min_quantity
			FROM '.DB_PREFIX.'orders_content oc
				INNER JOIN '.DB_PREFIX.'products p2 ON oc.pid = p2.pid
				INNER JOIN '.DB_PREFIX.'products_gifts g ON p2.pid = g.pid
				INNER JOIN '.DB_PREFIX.'products p ON g.gift_pid = p.pid AND p.is_visible = "Yes"
			WHERE oc.oid = '.intval($orderId)
		);
	}
}