<?php

class OrderProvider
{
	/**
	 * Refresh subtotals only
	 * 
	 * @param ORDER $order
	 */
	public static function refreshSubtotals(ORDER $order)
	{
		$order->checkShippingDigital();
		$order->recalcSubtotals();
	}

	/**
	 * Add item to an order
	 * 
	 * @param ORDER $order
	 * @param string $oa_id
	 * @param int $oa_quantity
	 * @param array $oa_attributes
	 *
	 * @return bool
	 */
	public static function addItem(ORDER $order, $oa_id, $oa_quantity, $oa_attributes = array())
	{
		return $order->addItem($oa_id, $oa_quantity, $oa_attributes);
	}

	/**
	 * Update order item
	 * 
	 * @param ORDER $order
	 * @param array $oa_quantity
	 * 
	 * @return bool
	 */
	public static function updateItems(ORDER $order, $oa_quantity)
	{
		$updateResult = true;

		if (count($oa_quantity) > 0)
		{
			$order->errors = array();
			$updateResult = $order->updateItems($oa_quantity);

			self::refreshSubtotals($order);
		}

		return $updateResult;
	}


	/**
	 * Remove order items
	 * 
	 * @param ORDER $order
	 * @param int $ocid
	 * @param $promo_code [description]
	 * 
	 * @return bool
	 */
	public static function removeItem(ORDER $order, $ocid, $promo_code = '')
	{
		$order->removeItem($ocid);

		$promo_result = self::applyPromo($order, $promo_code);

		self::refreshSubtotals($order);

		return $promo_result;
	}

	/**
	 * Clear order items
	 * 
	 * @param ORDER $order
	 */
	public static function clearItems(ORDER $order)
	{
		$order->clearItems();
		self::refreshSubtotals($order);
	}

	/**
	 * Apply promo code
	 * 
	 * @param ORDER $order
	 * @param string $promo_code
	 * @param boolean $forceReset
	 * 
	 * @return mixed
	 */
	public static function applyPromo(ORDER $order, $promo_code, $forceReset = false)
	{
		$promo_result = true;

		// Force reset when the user had the possiblity of updating the promo code
		if ($forceReset) $order->resetPromo();

		if (trim($promo_code) != '')
		{
			// Reset here when the promo code is just being checked for validity
			if (!$forceReset) $order->resetPromo();

			$promo_result = $order->checkPromoCode($promo_code);
		}

		return $promo_result;
	}


	public static function completeOrder($order, $user, $form, $customFields, $orderStatus = 'Process', $paymentStatus = 'Pending', $mobile = false, $orderSource = null, $offsiteCampaignId = null, $sendNotification = true)
	{
		$shipping_error_message = '';
		$order->recalcEverything($shipping_error_message);

		$validBilling = true;
		$validShipping = true;
		$validPayment = true;

		if (trim($user->data['fname'] == '') || trim($user->data['lname'] == ''))
		{
			$validBilling = false;
		}

		//Ensure shipping address loaded
		if (count($order->shippingAddress) == 0) $order->getShippingAddress();

		if ($order->shippingDigital != 1 &&
			(!isset($order->shippingAddress['name']) || trim($order->shippingAddress['name']) == '' || 
			 !isset($order->shippingAddress['address1']) || trim($order->shippingAddress['address1']) == ''))
		{
			$validShipping = false;
		}

		if ($order->paymentMethodName != 'Free' && ($order->paymentMethodId == 0 || $order->paymentIsRealtime != 'No'))
		{
			$validPayment = false;
		}

		$valid = $validPayment && $validShipping && $validBilling;

		if ($valid)
		{
			OrderProvider::closeOrder($order, $form, $customFields, $orderStatus, $paymentStatus,
				$mobile, $orderSource, $offsiteCampaignId, $sendNotification);
		}

		return $valid;
	}

	public static function closeOrder($order, $form, $customFields, $orderStatus = 'Process', $paymentStatus = 'Pending', $mobile = false, $orderSource = null, $offsiteCampaignId = null, $sendNotification = true)
	{
		$order->closeOrder(
			$orderStatus,
			$paymentStatus,
			$form,
			$customFields,
			$mobile,
			$orderSource,
			$offsiteCampaignId,
			$sendNotification
		);
	}

	/**
	 * Recalculate the order in the event of the user login (in case user level changes)
	 * 
	 * @param  ORDER $order
	 */
	public static function reupdateOrder($order, $resetDiscounts = true)
	{
		$order->reupdateItems();
		self::refreshSubtotals($order);
	}
}