<?php
	require_once("content/engine/payment/amazon/SignatureUtilsForOutbound.php");

	// Details for the payment method loader.
	$payment_processor_id = "amazon_sp";
	$payment_processor_name = "Amazon Simple Pay";
	$payment_processor_version = "1.0a";
	$payment_processor_class = "PAYMENT_AMAZON_SP";
	$payment_processor_type = "ipn";
		
	// Implementation Class
	class PAYMENT_AMAZON_SP extends PAYMENT_PROCESSOR
	{
		/** @var string AWS Secret & Access Keys */
		private $_secretKey = '';
		private $_accessKey = '';
		
		/**
	 	 * Sign a string using an AWS Secret Access Key
	 	 * @param string $stringToSign string to sign
	 	 * @return string signed string
	 	 */
		private function _signString($stringToSign)
		{
			$signature = hash_hmac('sha1', $stringToSign, $this->_AWS_SecretAccessKey);
			$signature = base64_encode(pack('H40', $signature));
			return $signature;
		}
	
		/**
		 * Create a string to sign based on an associative array of request parameters
		 * @param array $request associative array of request parameters
		 * @return string a string that needs to be signed
		 */
		private function _createSignatureString(array $data) 
		{
			$sigString = '';
			foreach ($data as $key => $value)
			{
				if (isset($value) && $key!='signature') $sigString .= ($sigString==""?"":"&").url_encode($key)."=".url_encode($value);
			}
			return $sigString;
		}
	
		/**
		 * Determine if an IPN POST is valid
		 * @param array $response an associative array of parameters returned from an IPN.  Typically you will just need to pass $_POST to this method
		 * @return bool true if the POST is valid, false if it is not
		 * @see http://docs.amazonwebservices.com/AmazonFPS/2007-01-08/AmazonSimplePayImplementationGuide/ipn.html
		 */
		public function isValid(array $response) 
		{
			uksort($response, 'strcasecmp');
			$generatedSignature = $this->_signString($this->_createSignatureString($response));
			return (strcmp($generatedSignature, $response['signature'])) ? false : true;
		}
		
		// Constructor
		function PAYMENT_AMAZON_SP()
		{
			// Initialise base class
			parent::PAYMENT_PROCESSOR();
				
			// Set up integration details
			$this->id = "amazon_sp";
			$this->name = "Amazon Simple Pay";
			$this->version = "1.0a";
			$this->class_name = "PAYMENT_AMAZON_SP";
			$this->type = "ipn";
			$this->description = "";
			$this->steps = 2;
			$this->testMode = false;
		}
		
		/**
		 * Turn an error code into a string
		 * @param $code
		 */
		function getErrorString($code)
		{
			$lookup = array(
				"PS"=>"Payment succeeded",
				"PF"=>"Payment failed",
				"RF"=>"Refund failed",
				"RS"=>"Refund succeeded",
				"ME"=>"Server error",
				"A"=>"Payment aborted",
				"SE"=>"Service error",
			);
			if(array_key_exists($code, $lookup))
				return $lookup[$code];
			
			return "Unknown error";
		}
		
		/**
		 * Generate form
		 * @param $db
		 */
		function getPaymentForm($db)
		{
			global $settings;
			$this->_secretKey = $this->settings_vars[$this->id."_secret_key"]["value"];
			$this->_accessKey = $this->settings_vars[$this->id."_access_key"]["value"];
			
			// Generate the default payment form. -- Ricky26
			$_pf = parent::getPaymentForm($db);
				
			// Work out URLs here, keeps code clean. -- Ricky26
			$abandon_url = $settings["GlobalHttpsUrl"]."/".$settings["INDEX"]."?pcsid=".session_id()."&_pcod=".$this->common_vars["order_security_id"]["value"]."&p=".$this->payment_page;
			$ipn_url = $return_url = $settings["GlobalHttpsUrl"]."/".$settings["INDEX"]."?pcsid=".session_id()."&_pcod=".$this->common_vars["order_security_id"]["value"]."&p=".$this->payment_page."&oa=".ORDER_PROCESS_PAYMENT;
			
			// Generate fields for amazon IN NATURAL ABC ORDER
			$data = array();
			$data["abandonUrl"] = $abandon_url;
			$data["accessKey"] = $this->settings_vars[$this->id."_access_key"]["value"];
			$data["amount"] = "USD " . number_format($this->common_vars["order_total_amount"]["value"], 2, ".", "");
			$data["description"] = $settings["GlobalSiteName"]." Order ".$this->common_vars["order_id"]["value"];
			$data["immediateReturn"] = "1";
			//$data["ipnUrl"] = $ipn_url;
			$data["processImmediate"] = "1";
			$data["referenceId"] = $this->common_vars["order_id"]["value"];
			$data["returnUrl"] = $return_url;
			$data["signatureVersion"] = 2;
			$data["signatureMethod"] = "HmacSHA1";
			
			$CanonicalizedQueryString = "";
			foreach ($data as $key=>$value)
			{
				$CanonicalizedQueryString .= ($CanonicalizedQueryString==""?"":"&").urlencode($key)."=".urlencode($value);
			}
			
			$StringToSign = 
				"GET\n".
				$this->isTestMode() ? "authorize.payments-sandbox.amazon.com\n" : "authorize.payments.amazon.com".
				"/pba/paypipeline\n".
				$CanonicalizedQueryString;
			
			$data["signature"] = base64_encode($this->hashHMAC($StringToSign, $this->_secretKey)); 
			
			$fields = array();
			
			foreach ($data as $key => $value)
			{
				$fields[] = array("type"=>"hidden", "name"=>$key, "value"=>$value);
			}
			
			return $fields;
		}
		
		/**
		 * Check is it a test mode
		 */
		function isTestMode()
		{
			$this->testMode = $this->settings_vars[$this->id."_test_mode"]['value'] != "No";
			return $this->testMode;
		}
		
		/**
		 * Get JS code for form validation
		 */ 
		function getValidatorJS()
		{
			return "
			if ($('.payment-agree:visible').length && $('.payment-agree:visible').is(':checked'))
			{
				$('.payment-agree:visible').attr('checked', false);
				return true;
			}";
		}
		
		/**
		 * Get the url to forward to.
		 */
		function getPostUrl()
		{
			if ($this->url_to_gateway != "")
			{
				$this->post_url = trim($this->url_to_gateway);
			}	
			elseif ($this->isTestMode())
			{
				$this->post_url = "https://authorize.payments-sandbox.amazon.com/pba/paypipeline";
			}
			else 
			{
				$this->post_url = "https://authorize.payments.amazon.com/pba/paypipeline";	
			}
		
			return $this->post_url;
		}
		
		// Process the result from the AWS.
		function process($db, $user, $order, $post_form)
		{			
			global $settings;
			$_DATA = $_GET;
			
			$this->_secretKey = $this->settings_vars[$this->id."_secret_key"]["value"];
			$this->_accessKey = $this->settings_vars[$this->id."_access_key"]["value"];
			
			$signatureVersion = isset($_DATA["signatureVersion"]) ? $_DATA["signatureVersion"] : false;
			$signatureMethod = isset($_DATA["signatureMethod"]) ? $_DATA["signatureMethod"] : false;
			$certificateUrl = isset($_DATA["certificateUrl"]) ? $_DATA["certificateUrl"] : false;
			$signature = isset($_DATA["signature"]) ? $_DATA["signature"] : false;
			
			if ($signatureVersion == 2 && $signatureMethod == "RSA-SHA1" && $certificateUrl && $signature)
			{
				$utils = new Cts_Amazon_FPS_SignatureUtilsForOutbound($this->_accessKey, $this->_secretKey);
				
				$urlEndPoint = $settings["GlobalHttpsUrl"]."/".$settings["INDEX"]."?pcsid=".session_id()."&_pcod=".$this->common_vars["order_security_id"]["value"]."&p=".$this->payment_page."&oa=".ORDER_PROCESS_PAYMENT;
				
				$verificationResult = $utils->validateRequest($_DATA, $urlEndPoint, "GET");
				
				if ($verificationResult)
				{
					if ($_DATA["status"] == "PS")
					{
						$response = "";
						foreach ($_DATA as $key=>$value) $response.=$key." = ".$value."\n";
						
						$this->createTransaction($db, $user, $order, $response);
					}
					else
					{
						$this->is_error = true;
						$this->error_message = $this->getErrorString($_DATA["status"]);
					}
				}
				else
				{
					$this->is_error = true;
					$this->error_message = "Cannot verificate request";
				}
			}
			else
			{
				$this->is_error = true;
				$this->error_message = "One or more parameters missing";
			}
			return !$this->is_error;
		}
		
		// Generate a HMAC hash.
		function hashHMAC($data, $key)
		{
			$sig = hash_hmac("sha1", $data, $key, TRUE);
			return $sig;
		}
	}
